from models import Project, Category, Department, Status
from django.contrib.admin import widgets
from django import forms
from django.forms import ModelForm
from django.contrib.auth.models import User

class ProjectInitiationForm(ModelForm):		# Extends ModelForm because it's based on a model
	smalltextarea=forms.Textarea({'rows': 4, 'cols': 35,})	# A few text area widgets we're gonna reuse later
	largetextarea=forms.Textarea({'rows': 6, 'cols': 35,})

	title=forms.CharField(
		required=True,
		max_length=100,
		label="Title *",
	)

	category=forms.ModelChoiceField(
		widget=forms.Select,
		queryset=Category.objects.filter(is_active=True).order_by('name',),
		required=True,
		label="Task Category *",
	)

	date_needed=forms.DateField(
		widget=forms.TextInput(),
		required=True,
		label="Date Needed *",
	)

	department=forms.ModelChoiceField(
		widget=forms.Select,
		queryset=Department.objects.filter(is_active=True).order_by('name'),
		required=True,
		label="Department *",
	)

	contact_person=forms.ModelChoiceField(
		widget=forms.Select(),
		queryset=User.objects.order_by('last_name', 'first_name'),	# This actually gets redefined in the __init__ below...
		required=True,
		label="Contact Person *",
		help_text='(The person who is most knowledgeable about the business problem.)',
	)
	hardware_needed=forms.CharField(widget=smalltextarea, required=False)
	software_needed=forms.CharField(widget=smalltextarea, required=False)
	people_needed=forms.CharField(
		widget=smalltextarea, 
		required=False,
		help_text='(Incremental human resources such as consultants, vendors, or staff required for implementation or ongoing support.)',
	)
	performance_measurement=forms.CharField(
		widget=largetextarea, 
		label='Performance Measurement *',
		required=True,
		help_text='(How will you know the project was a success?)',
	)
	description=forms.CharField(widget=largetextarea, required=True, label="Description *")
	
	class Meta:	# Meta subclass contains options for the form that are not actually *part* of the form
		model = Project	# This form is based on the "Project" model... 
		exclude = ('originator', 'priority', 'status', 'is_active')	# Don't show these by default
	
	class Media:	# Media subclass contains links to media (css, javascript, etc) that need to be brought in by the form
		js = (
			'jquery/js/jquery-1.3.2.min.js',
			'jquery/js/jquery-ui-1.7.2.custom.min.js',
		)
		css = {
			'all': ('jquery/css/ui-lightness/jquery-ui-1.7.2.custom.css',)
		}

	# Here, we are overriding the __init__ for this form to allow the "contact person" field to display friendlier options (since User's __unicode__ is just the username)
	# See http://code.djangoproject.com/ticket/6839
	def __init__(self, *args, **kwargs):
		super(ProjectInitiationForm, self).__init__(*args, **kwargs)
		self.fields['contact_person'].queryset = User.objects.order_by('last_name', 'first_name')
		self.fields['contact_person'].label_from_instance = lambda obj: "%s, %s (%s)" % (obj.last_name, obj.first_name, obj.username)



class ReportForm(forms.Form):	# The form for report generation is *not* based on a model, so it doesn't extend ModelForm or have a Meta class
	ORDER_CHOICES = (
		('id', 'Project ID',),
		('category', 'Category',),
		('department', 'Department',),
	)

	category=forms.ModelChoiceField(
		widget=forms.Select,
		queryset=Category.objects.order_by('name',),
		required=False,
		label="Category",
	)
	status=forms.ModelChoiceField(
		widget=forms.Select,
		queryset=Status.objects.order_by('name',),
		required=False,
		label="Status",
	)
	department=forms.ModelChoiceField(
		widget=forms.Select,
		queryset=Department.objects.order_by('name',),
		required=False,
		label="Department",
	)
	order_by=forms.ChoiceField(
		choices=ORDER_CHOICES,
		initial='id',
		required=False,
	)

	include_inactive_projects=forms.BooleanField(
		required=False,
	)

	include_comments=forms.BooleanField(
		required=False,
		initial=False,
	)
