# Model Definitions. These form the "M" in "MTV" or "MVC"
from django.db import models
from django.contrib.auth.models import User

class Status(models.Model):	# Create a "Status" model / database table
	name = models.CharField(max_length=25)	# "Name" is a required, 25-character-or-less Character Field
	description = models.TextField(blank=True)	# "Description" is an optional Text Field
	is_active = models.BooleanField(blank=False, null=False, default=True)	# Instead of deleting things, we can mark them as inactive, but they default to active
	class Meta:	# This subclass contains "Meta" options for the model (i.e. things about the Model, that aren't *part* of the Model)
		ordering = ['name']	# When we create an iterable (queryset, or whatever), sort objects by the "name" field (also affects the Admin)
		verbose_name_plural='statuses'	# Django pluralizes by adding an "s" to the end of things. But we don't want "statuss"
	def __unicode__(self):
		return self.name	# When django refers to an instance of a Status object, it will display the instance's "name" field

class Category(models.Model):	# Create a "Category" model / database table
	name = models.CharField(max_length=50)	# "Name" is a required, 50-character-or-less Character Field
	description = models.TextField(blank=True)	# "Description" is an optional Text Field
	is_active = models.BooleanField(blank=False, null=False, default=True)	# Instead of deleting things, we can mark them as inactive, but they default to active
	class Meta:	# This subclass contains "Meta" options for the model (i.e. things about the Model, that aren't *part* of the Model)
		ordering = ['name']	# When we create an iterable (queryset, or whatever), sort objects by the "name" field (also affects the Admin)
		verbose_name_plural='categories'	# Django pluralizes by adding an "s" to the end of things. But we don't want "categorys"
	def __unicode__(self):
		return self.name	# When Django refers to an instance of a Category object, it will display the instance's "name" field

class Department(models.Model):	# Create a "Department" model / database table
	name = models.CharField(max_length=50)	# "Name" is a required, 50-character-or-less Character Field
	is_active = models.BooleanField(blank=False, null=False, default=True)	# Instead of deleting things, we can mark them as inactive, but they default to active
	class Meta:	# This subclass contains "Meta" options for the model (i.e. things about the Model, that aren't *part* of the Model)
		ordering = ['name']	# When we create an iterable (queryset, or whatever), sort objects by the "name" field (also affects the Admin)
	def __unicode__(self):
		return self.name	# When Django refers to a "Department" object, it will display the instance's "name" field

class Project(models.Model):	# Create a "Project" model / database table
	PRIORITY_CHOICES = (		# These are the choices that will be used for the "priority" field below.
		(0, 'Not Assigned',),	# The first position of each tuple is stored in the database.
		(1, 'Low',),		# The second position of each tuple is displayed with the builtin method get_priority_display()
		(2, 'Medium',),		# If we had "FOO_CHOICES", there would be a "get_foo_display()
		(3, 'High',),		# get_foo_display() is called automatically by things like Django's admin
		(4, 'Extremely Urgent',),
	)
	title = models.CharField(max_length=100)	# "Title" is a required, 100-character-or-less Character Field
	category = models.ForeignKey(Category)		# "Category" is a required field, and is a Foreign Key to the "Category" model
	priority = models.IntegerField(default=0, choices=PRIORITY_CHOICES)	# "Priority" is a required Integer Field, that uses PRIORITY_CHOICES as its choices. "0", or "Not Assigned" is the default
	description = models.TextField(blank=True)	# "Description" is a blank Text Field
	cost_estimate = models.CharField(max_length=255, blank=True)	# "Cost Estimate" is an optional, 255-character-or-less Character Field
	date_requested = models.DateTimeField(auto_now_add=True, editable=False)	# "Date Requested" is a DateTimeField that gets filled in with the timestamp when a model instance is created. We can't edit this in the Admin.
	date_modified = models.DateTimeField(auto_now=True, editable=False)	# "Date Modified" is a DateTimeField that gets updated with the timestamp when a model instance is saved / updated. We can't edit this in the Admin.
	date_needed = models.DateField()	# "Date Needed" is a required DateField
	department = models.ForeignKey(Department)	# "Department" is a required field, and is a Foreign Key to the "Department" model
	status = models.ForeignKey(Status)	# "Status" is a required field, and is a Foreign Key to the "Status" model
	originator = models.ForeignKey(User, related_name='originator')	# "Originator": The person who created the project. It's required, and a foreign key to Django's auth User model. Since we have more than one FK to that model, it's got a "related name"
	its_contact = models.ForeignKey(User, related_name='its_contact', blank=True, null=True) # "ITS Contact": The person in ITS that people should contact about this project. It's optional, can be null, and is a foreign key to Django's auth User model. Since we have more than one FK to that model, it's got a "related name"
	contact_person = models.ForeignKey(User, related_name='contact_person') # "Contact Person": The person that people should contact about this project. It's required, and is a foreign key to Django's auth User model. Since we have more than one FK to that model, it's got a "related name"
	hardware_needed = models.TextField(blank=True)	# Optional text field where people can specify the hardware needs of their project
	software_needed = models.TextField(blank=True)	# Optional text field where people can specify the software needs of their project
	people_needed = models.TextField(blank=True)	# Optional text field where people can specify the personnel needs of their project
	performance_measurement = models.TextField()	# Required text field for people to input performance and assessment information on how they will know their project was a success
	programmer_assigned = models.ForeignKey(User, related_name='programmer_assigned', blank=True, null=True,)	# "Programmer Assigned": Some projects require developer resources. It's an optional field that can be null, and since we're using a FK to the "User" model, it has a "related name"
	is_active = models.BooleanField(blank=False, null=False, default=True) # Instead of deleting old projects we can mark them as inactive. But the default is "active"
	class Meta:	# This subclass contains "Meta" options for the model (i.e. things about the Model, that aren't *part* of the Model)
		ordering = ['-date_requested']	# When we create an interable (queryset, or whatever), sort objects descending by the date_requested
	def __unicode__(self):	# When django refers to an instance of a Project, it will say something like "Project ID #1 requested by Smith, Steven (ssmith46) on 2009-10-05."
		return "Project ID #%s (%s) requested by %s on %s." %(self.id, self.title, self.originator, self.date_requested)
	def get_absolute_url(self):	# This is the url (in this case, the public view of the project), that will get returned in things like RSS Feeds. Don't forget to update the value for "domain name" in the admin, under "sites", otherwise you'll have example.com!
		return "/view/%d/" % self.id
